<?php

$config = [
    'encryption_password' => 'test_passwd',
];

error_reporting(E_ALL);
ini_set('display_errors', 0);
ini_set('log_errors', 1);
ini_set('error_log', 'error.log');
set_time_limit(0);
ini_set('zlib.output_compression', 'Off');
ini_set('output_buffering', 'Off');
header("Content-Type: application/octet-stream");
header_remove("X-Powered-By");
header_remove("Transfer-Encoding");
header_remove("Content-Encoding");

if (ob_get_level()) ob_end_clean();
ob_implicit_flush(true);

define('TMP_DIR', sys_get_temp_dir() . '/websocks_sessions');
if (!file_exists(TMP_DIR)) {
    mkdir(TMP_DIR, 0700, true);
}

function log_debug($message) {
    error_log("[SOCKS5-PROXY] " . $message);
}

class Encryptor {
    private $key;
    private $salt;
    private $enabled;
    public function __construct($password, $salt = null) {
        $this->enabled = !empty($password);

        if (!$this->enabled) {
            return;
        }

        $this->salt = $salt ? base64_decode($salt) : openssl_random_pseudo_bytes(16);

        $this->key = hash_pbkdf2(
            'sha256',
            $password,
            $this->salt,
            100000,
            32,
            true
        );
    }

    public function encrypt($data) {
        if (!$this->enabled || empty($data)) {
            return $data;
        }
        $iv = openssl_random_pseudo_bytes(16);
        $padSize = 16 - (strlen($data) % 16);
        $data = $data . str_repeat(chr($padSize), $padSize);

        $encrypted = openssl_encrypt(
            $data,
            'AES-256-CBC',
            $this->key,
            OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING,
            $iv
        );

        return $iv . $encrypted;
    }

    public function decrypt($data) {
        if (!$this->enabled || empty($data) || strlen($data) <= 16) {
            return $data;
        }
        $iv = substr($data, 0, 16);
        $encrypted = substr($data, 16);
        $decrypted = openssl_decrypt(
            $encrypted,
            'AES-256-CBC',
            $this->key,
            OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING,
            $iv
        );
        if ($decrypted === false) {
            log_debug("encrypted failed");
            return '';
        }
        $padSize = ord(substr($decrypted, -1));
        if ($padSize < 1 || $padSize > 16) {
            log_debug("Invalid fill size: " . $padSize);
            return '';
        }
        return substr($decrypted, 0, -$padSize);
    }

    public function getSaltBase64() {
        if (!$this->enabled) {
            return '';
        }
        return base64_encode($this->salt);
    }

    public function isEnabled() {
        return $this->enabled;
    }
}

$action = $_GET['action'] ?? $_POST['action'] ?? '';
$stream_id = $_GET['stream_id'] ?? $_POST['stream_id'] ?? '';
$target = $_POST['target'] ?? '';
$port = intval($_POST['port'] ?? 0);
$encrypted = isset($_POST['encrypted']) && $_POST['encrypted'] == 1;
$salt = $_POST['salt'] ?? null;
log_debug("Request: $action, Stream ID: $stream_id, Target: $target:$port, Encrypted: " . ($encrypted ? "Yes" : "No"));
$meta_file = TMP_DIR . "/sock_$stream_id.meta";
$data_file = TMP_DIR . "/sock_{$stream_id}.data";

try {
    if (empty($stream_id)) {
        log_debug("Missing stream_id");
        http_response_code(400);
        echo "Missing stream_id";
        exit;
    }

    switch ($action) {
        case 'connect':
            if (empty($target) || $port < 1 || $port > 65535) {
                log_debug("Invalid target or port");
                http_response_code(400);
                echo "Invalid target or port";
                exit;
            }

            $encryptor = null;
            if ($encrypted && !empty($config['encryption_password'])) {
                $encryptor = new Encryptor($config['encryption_password'], $salt);
                log_debug("Encryption enabled with salt: " . $salt);
            } else {
                log_debug("Encryption not enabled for this connection");
            }

            log_debug("Connecting to $target:$port");
            $socket = @fsockopen($target, $port, $errno, $errstr, 5);

            if (!$socket) {
                log_debug("Connection failed: $errstr ($errno)");
                http_response_code(502);
                echo "Connect failed: $errstr ($errno)";
                exit;
            }

            stream_set_blocking($socket, false);

            $meta = [
                'target' => $target,
                'port' => $port,
                'created' => time(),
                'last_activity' => time(),
                'encrypted' => $encrypted,
                'salt' => $salt
            ];

            file_put_contents($meta_file, json_encode($meta));
            log_debug("Connection established to $target:$port");
            fclose($socket);
            echo "OK";
            break;

        case 'send':
            if (!file_exists($meta_file)) {
                log_debug("Session not found for send");
                http_response_code(404);
                echo "Session not found";
                exit;
            }

            $meta = json_decode(file_get_contents($meta_file), true);
            $target = $meta['target'];
            $port = $meta['port'];
            $encrypted = isset($meta['encrypted']) && $meta['encrypted'];
            $salt = $meta['salt'] ?? null;

            $data = file_get_contents("php://input");
            log_debug("Received " . strlen($data) . " bytes from client");

            if ($encrypted && !empty($config['encryption_password'])) {
                $encryptor = new Encryptor($config['encryption_password'], $salt);
                $data = $encryptor->decrypt($data);
                log_debug("Decrypted data: " . strlen($data) . " bytes");
            }

            $socket = @fsockopen($target, $port, $errno, $errstr, 5);

            if (!$socket) {
                log_debug("Reconnection failed for send: $errstr ($errno)");
                http_response_code(502);
                echo "Send failed: $errstr ($errno)";
                exit;
            }

            $bytes_written = fwrite($socket, $data);
            fflush($socket);

            $meta['last_activity'] = time();
            file_put_contents($meta_file, json_encode($meta));

            $response_data = '';
            stream_set_blocking($socket, false);

            for ($i = 0; $i < 5; $i++) {
                $read = [$socket];
                $write = $except = null;

                if (stream_select($read, $write, $except, 0, 200000)) {
                    while (!feof($socket)) {
                        $chunk = fread($socket, 8192);
                        if ($chunk === false || $chunk === '') {
                            break;
                        }
                        $response_data .= $chunk;

                        if (strlen($response_data) > 16384 || strpos($response_data, "\r\n\r\n") !== false) {
                            break;
                        }
                    }

                    if (!empty($response_data)) {
                        if ($encrypted && !empty($config['encryption_password'])) {
                            $encryptor = new Encryptor($config['encryption_password'], $salt);
                            $response_data = $encryptor->encrypt($response_data);
                            log_debug("Encrypted response: " . strlen($response_data) . " bytes");
                        }

                        file_put_contents($data_file, $response_data);
                        log_debug("Got immediate response: " . strlen($response_data) . " bytes");
                        break;
                    }
                }

                usleep(50000); // 50ms
            }
            fclose($socket);
            log_debug("Sent $bytes_written bytes to $target:$port");
            echo "OK ($bytes_written bytes)";
            break;

        case 'recv':
            if (!file_exists($meta_file)) {
                log_debug("Session not found for recv");
                http_response_code(404);
                echo "Session not found";
                exit;
            }

            $meta = json_decode(file_get_contents($meta_file), true);
            $encrypted = isset($meta['encrypted']) && $meta['encrypted'];
            $salt = $meta['salt'] ?? null;
            $target = $meta['target'];
            $port = $meta['port'];

            if (file_exists($data_file)) {
                $data = file_get_contents($data_file);
                unlink($data_file);
                log_debug("Returning previously stored data: " . strlen($data) . " bytes");
                $meta['last_activity'] = time();
                file_put_contents($meta_file, json_encode($meta));
                echo $data;
                exit;
            }
            log_debug("Connecting to $target:$port for recv");
            $socket = @fsockopen($target, $port, $errno, $errstr, 5);

            if (!$socket) {
                log_debug("Reconnection failed for recv: $errstr ($errno)");
                http_response_code(502);
                echo "Recv failed: $errstr ($errno)";
                exit;
            }

            stream_set_blocking($socket, false);
            $data = '';
            $start_time = microtime(true);
            $timeout = 2.0; // 2 seconds timeout

            while ((microtime(true) - $start_time) < $timeout) {
                $read = [$socket];
                $write = $except = null;

                if (stream_select($read, $write, $except, 0, 200000)) { // 200ms
                    $chunk = fread($socket, 8192);
                    if ($chunk === false) {
                        break;
                    }
                    if ($chunk === '') {
                        usleep(10000); // 10ms
                        continue;
                    }

                    $data .= $chunk;
                    if (strlen($data) > 0) {
                        $timeout = microtime(true) + 0.2;
                    }

                    if (strlen($data) > 65536) {
                        break;
                    }
                } else {
                    if (strlen($data) > 0) {
                        break;
                    }
                    usleep(10000); // 10ms
                }
            }

            fclose($socket);
            if (!empty($data) && $encrypted && !empty($config['encryption_password'])) {
                $encryptor = new Encryptor($config['encryption_password'], $salt);
                $data = $encryptor->encrypt($data);
                log_debug("Encrypted recv data: " . strlen($data) . " bytes");
            }
            if (!empty($data)) {
                log_debug("Received " . strlen($data) . " bytes from $target:$port");
                $meta['last_activity'] = time();
                file_put_contents($meta_file, json_encode($meta));
            } else {
                log_debug("No data received from $target:$port");
            }
            echo $data;
            break;

        case 'close':
            log_debug("Closing connection for stream $stream_id");
            @unlink($meta_file);
            @unlink($data_file);
            echo "CLOSED";
            break;

        default:
            log_debug("Default response for detection");
            echo "<!-- " . base64_encode(random_bytes(10)) . " -->";
            break;
    }
} catch (Exception $e) {
    log_debug("Error: " . $e->getMessage());
    http_response_code(500);
    echo "Error: " . $e->getMessage();
}

if (mt_rand(1, 10) === 1) {
    log_debug("Cleaning old sessions");
    foreach (glob(TMP_DIR . "/sock_*.meta") as $file) {
        if (time() - filemtime($file) > 3600) {
            @unlink($file);
            $data_file = str_replace('.meta', '.data', $file);
            if (file_exists($data_file)) {
                @unlink($data_file);
            }
        }
    }
}

